/******************************************************************************/
/* Configuration pour l'archivage des diffrents lments du fichier source   */
/******************************************************************************/
// !CONFIG!=/L/* /R/* /W"* Nom : "
// Dfinition du systme       !CONFIG!=/V1!EMULATEUR CPC!
// Dfinition du sous systme  !CONFIG!=/V2!PC-CPC!
// Dfinition du sous ensemble !CONFIG!=/V3!Chips!
// Dfinition du module        !CONFIG!=/V4!CRTC 6845!
/******************************************************************************/

/********************************************************* !NAME! **************
* !./FLE!
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\Fichiers
********************************************************** !0! *****************
* ------------------------------------------------------------------------------
*          SYSTEME         |      SOUS SYSTEME       |      SOUS ENSEMBLE
* ------------------------------------------------------------------------------
*  EMULATEUR CPC           | PC-CPC                  | Chips
* ------------------------------------------------------------------------------
*  Fichier     : CRTC.C                | Version : 0.1an
* ------------------------------------------------------------------------------
*  Date        : 05/11/2002            | Auteur  : L.DEPLANQUE
* ------------------------------------------------------------------------------
*  Description : Emulation du CRTC 6845
*
* ------------------------------------------------------------------------------
*  Historique  :
*           Date           |         Auteur          |       Description
* ------------------------------------------------------------------------------
*  05/11/2002              | L.DEPLANQUE             | creation
* ------------------------------------------------------------------------------
*  13/11/2002              | L.DEPLANQUE             | Changement du type de
*                          |                         | la variable TabAdrCrtc : 
*                          |                         | passage en USHORT, pour
*                          |                         | optimiser le memset
*                          |                         | dans CalcAdrEcrCPC()
* ------------------------------------------------------------------------------
*  08/02/2006              | L.DEPLANQUE             | Version 0.1ak :
*                          |                         | Rcriture complte du
*                          |                         | code
* ------------------------------------------------------------------------------
*  06/12/2006              | L.DEPLANQUE             | Version 0.1am :
*                          |                         | Variable "VBL" remplace
*                          |                         | par "VSync" et supprime
*                          |                         | du module PPI.
*                          |                         | Correction problme de
*                          |                         | synchro multiples.
* ------------------------------------------------------------------------------
*  01/03/2007              | L.DEPLANQUE             | Version 0.1an :
*                          |                         | Rcriture complte de
*                          |                         | la fonction principale
*                          |                         | du CRTC, prise en compte
*                          |                         | d'un cycle (1 mot vido)
*                          |                         | au lieu d'une ligne HBL
* ------------------------------------------------------------------------------
********************************************************** !END! **************/


#include  <windows.h>
#include  <stdio.h>

#include  "Types.h"
#include  "PlateForm.h"
#include  "VGA.H"
#include  "Z80.h"
#ifdef USE_LOG
#include  "Log.h"
#endif


#define     MODULENAME      "CRTC"


#define     TAILLE_X_ECR    384
#define     TAILLE_Y_ECR    272

#define     TAILLE_Y_CRTC   312     // Le crtc gnre 312 lignes verticales

#define     TAILLE_VBL      26      // VBL = 26 lignes


static int x, y, LigneCRTC, SyncCount, CptCharVert, NumLigneChar, MaCRTC, MaHi;

static int TailleVBL, TailleHBL, HDelay, VDelay, VswCount;

static int HDT, VDT, HS, MR, VT, OldVSync, VtAdj;

static BYTE VCharCount, VHCount, CRTCCharCount;


/********************************************************* !NAME! **************
* Nom : RegsCRTC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Registres du CRTC
*
********************************************************** !0! ****************/
int RegsCRTC[ 32 ];


/********************************************************* !NAME! **************
* Nom : RegCRTCSel
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro du registre CRTC slectionn
*
********************************************************** !0! ****************/
int RegCRTCSel;


/********************************************************* !NAME! **************
* Nom : CntHSync
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Compteur position horizontale raster
*
********************************************************** !0! ****************/
int CntHSync = 0;


/********************************************************* !NAME! **************
* Nom : VSync
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Signal VSync
*
********************************************************** !0! ****************/
int VSync;


/********************************************************* !NAME! **************
* Nom : DoResync
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique qu'il faut synchroniser l'affichage
*
********************************************************** !0! ****************/
int DoResync;


/********************************************************* !NAME! **************
* Nom : ResetCRTC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Effectue un reset du CRTC
*
* Rsultat    : /
*
* Variables globales modifies : RegsCRTC
*
********************************************************** !0! ****************/
void ResetCRTC( void )
{
    x = y = LigneCRTC = 0;
    SyncCount = CptCharVert = NumLigneChar = MaCRTC = MaHi = 0;
    TailleVBL = TailleHBL = 16;
    HDelay = VDelay = VswCount = 0;
    HDT = VDT = 0;
    HS = MR = VT = OldVSync = VtAdj = 0;
    VCharCount = VHCount = CRTCCharCount = 0;
    memset( RegsCRTC, -1, sizeof( RegsCRTC ) );
}


/********************************************************* !NAME! **************
* Nom : ReadCRTC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Lecture d'un registre du CRTC
*
* Rsultat    : Le registre du CRTC selectionn
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
int ReadCRTC( int adr )
{
    adr &= 0xBF00;
    if ( adr == 0xBF00 )
        return( ( RegCRTCSel >= 12 ) ? RegsCRTC[ RegCRTCSel ] : 0 );

    if ( adr == 0xBE00 )
        return( VSync << 5 );       // Reg. Status sur Crtc Type 1 seulement ???

    return( 0xFF );
}


/********************************************************* !NAME! **************
* Nom : WriteCRTC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ecriture d'un registre du CRTC
*
* Rsultat    : /
*
* Variables globales modifies : RegsCRTC, RegCRTCSel, XStart, XEnd
*
********************************************************** !0! ****************/
void WriteCRTC( int adr, int val )
{
    adr &= 0xBF00;
    if ( adr == 0xBC00 )
        RegCRTCSel = val & 0x1F;
    else
        if ( adr == 0xBD00 )
            {
            switch( RegCRTCSel )
                {
                case 4 :
                case 6 :
                case 7 :
                    val &= 0x7F;
                    break;

                case 5 :
                case 9 :
                    val &= 0x1F;
                    break;

                case 12 :
                    MaHi = ( ( val << 8 ) & 0x0F00 ) | ( ( val << 9 ) & 0x6000 );
                    break;
                }
            RegsCRTC[ RegCRTCSel ] = val;
            }
}


/********************************************************* !NAME! **************
* Nom : CycleCRTC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Effectue n cycles CRTC, trace les mots en mmoire vido
*
* Rsultat    : /
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
void CycleCRTC( int NbCycles )
{
    int CalcMa = 0;

    for ( ; NbCycles--; )
        {
        VCharCount++;
        if ( CRTCCharCount++ == RegsCRTC[ 0 ] )
            {
            CRTCCharCount = 0;
            NumLigneChar = ( NumLigneChar + 1 ) & 0x1F;
            if ( VSync )
                {
                if ( ++TailleVBL >= RegsCRTC[ 3 ] >> 4 )
                    {
                    OldVSync = 1;
                    VSync = 0;
                    }
                }
            if ( MR )
                {
                MR = 0;
                NumLigneChar = 0;
                if ( ! HDT )
                    MaCRTC += RegsCRTC[ 1 ];

                CptCharVert = ( CptCharVert + 1 ) & 0x7F;
                }
            if ( VtAdj )
                CalcMa = ( ! --VtAdj );

            if ( VT )
                {
                VT = 0;
                VtAdj = RegsCRTC[ 5 ];
                CalcMa = ! VtAdj;
                }
            if ( CalcMa )
                {
                CalcMa = OldVSync = NumLigneChar = CptCharVert = 0;
                VDT = 1;
                MaCRTC = RegsCRTC[ 13 ] + MaHi;
                }

            if ( NumLigneChar == RegsCRTC[ 9 ] )
                {
                MR = 1;
                if ( ! VtAdj && CptCharVert == RegsCRTC[ 4 ] )
                    VT = 1;
                }
            if ( CptCharVert == RegsCRTC[ 6 ] )
                VDT = 0;

            if ( CptCharVert == RegsCRTC[ 7 ] && ! VSync && ! OldVSync )
                {
                TailleVBL = 0;
                SyncCount = 2;
                VSync = 1;
                VDelay = 2;
                VswCount = 4;
                }
            HDT = 1;
            }
        if ( CRTCCharCount == RegsCRTC[ 1 ] )
            HDT = 0;

        if ( HS )
            {
            if ( HDelay == 2 )
                {
                if ( ! --VHCount )
                    {
                    LigneCRTC++;
                    VCharCount = 0;
                    HDelay++;
                    }
                }
            else
                HDelay++;
            }

        if ( ! --TailleHBL )
            {
            HS = 0;
            if ( VDelay )
                VDelay--;

            if ( ! VDelay )
                {
                if ( VswCount )
                    {
                    if ( ! --VswCount )
                        {
                        LigneCRTC = 0;
                        DoResync = 1;
                        }
                    }
                }
            CntHSync++;
            if ( SyncCount )
                {
                if ( ! --SyncCount )
                    {
#ifdef USE_DESASM
                    if ( ! DebugMode )
#endif
                        IRQ = CntHSync & 32;

                    CntHSync = 0;
                    }
                }
            //
            // Si 52 lignes compte -> Gnration d'une interruption
            //
            if ( CntHSync == 52 )
                {
                CntHSync = 0;
#ifdef USE_DESASM
                if ( ! DebugMode )
#endif
                    IRQ = 1;
                }
            LastMode = MemoMode; // Prise en compte nouveau mode cran
            }
        if ( CRTCCharCount == RegsCRTC[ 2 ] )
            {
            HDelay = 0;
            HS = 1;
            TailleHBL = ( BYTE )( RegsCRTC[ 3 ] & 0x0F );
            VHCount = ( BYTE )max( 0, min( ( TailleHBL - 2 ), 4 ) );
            if ( ! TailleHBL )
                TailleHBL = 16;
            }
        if ( y && x < TAILLE_X_ECR )
            {
            int Adr = -1;
            if ( HDT && VDT )
                {
                Adr = ( CRTCCharCount + MaCRTC ) << 1;
                if ( Adr & 0x2000 )
                    Adr += 0x4000;
                }
            DrawFct( x
                   , y
                   , ( Adr & 0xFFFFC7FF ) | ( NumLigneChar << 11 )
                   , LigneCRTC < TAILLE_VBL
                   );
            x += 8;
            }
        else
            {
            if ( VCharCount == 7 )
                {
                if ( y < TAILLE_Y_ECR )
                    {
                    x = 0;
                    y++;
                    }
                else
                    if ( LigneCRTC == TAILLE_VBL )
                        y = 0;
                }
            if ( LigneCRTC == TAILLE_Y_CRTC )
                LigneCRTC = 0;
            }
        }
}
