/******************************************************************************/
/* Configuration pour l'archivage des diffrents lments du fichier source   */
/******************************************************************************/
// !CONFIG!=/L/* /R/* /W"* Nom : "
// Dfinition du systme       !CONFIG!=/V1!EMULATEUR CPC!
// Dfinition du sous systme  !CONFIG!=/V2!PC-CPC!
// Dfinition du sous ensemble !CONFIG!=/V3!Chips!
// Dfinition du module        !CONFIG!=/V4!PPI 8255!
/******************************************************************************/

/********************************************************* !NAME! **************
* !./FLE!
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\Fichiers
********************************************************** !0! *****************
* ------------------------------------------------------------------------------
*          SYSTEME         |      SOUS SYSTEME       |      SOUS ENSEMBLE
* ------------------------------------------------------------------------------
*  EMULATEUR CPC           | PC-CPC                  | Chips
* ------------------------------------------------------------------------------
*  Fichier     : PPI.C                 | Version : 0.1am
* ------------------------------------------------------------------------------
*  Date        : 05/11/2002            | Auteur  : L.DEPLANQUE
* ------------------------------------------------------------------------------
*  Description : Emulation du PPI 8255
*
* ------------------------------------------------------------------------------
*  Historique  :
*           Date           |         Auteur          |       Description
* ------------------------------------------------------------------------------
*  05/11/2002              | L.DEPLANQUE             | creation
* ------------------------------------------------------------------------------
*  06/11/2002              | L.DEPLANQUE             | Prcisions bits du port
*                          |                         | B (Busy, 50Hz refresh)
* ------------------------------------------------------------------------------
*  07/11/2002              | L.DEPLANQUE             | Ajout gestion lecture et
*                          |                         | criture cassette.:
*                          |                         | fonctions WriteCas() et
*                          |                         | ReadCas().
* ------------------------------------------------------------------------------
*  13/11/2002              | L.DEPLANQUE             | Optimisation WriteCas() 
*                          |                         | et ReadCas() : supprime
*                          |                         | test pointeur du fichier
*                          |                         | car inclus dans la
*                          |                         | variable Mode
* ------------------------------------------------------------------------------
*  03/12/2002              | L.DEPLANQUE             | Suppression de la
*                          |                         | gnration de sons lors
*                          |                         | de la lecture/criture
*                          |                         | cassette
* ------------------------------------------------------------------------------
*  09/01/2003              | L.DEPLANQUE             | Version 0.1u :
*                          |                         | corection d'un bugg :
*                          |                         | inversion des bits 
*                          |                         | PRINTER_BUSY et
*                          |                         | REFRESH_HZ
* ------------------------------------------------------------------------------
*  09/01/2004              | L.DEPLANQUE             | Version 0.1x :
*                          |                         | corection d'un bugg :
*                          |                         | problme d'criture du
*                          |                         | buffer cassette dans le
*                          |                         | fichier correspondant
*                          |                         | (crivait une taille=0)
* ------------------------------------------------------------------------------
*  02/02/2004              | L.DEPLANQUE             | Ajout de l'mission d'un
*                          |                         | son lors de la lecture
*                          |                         | ou l'criture sur
*                          |                         | cassette.
* ------------------------------------------------------------------------------
*  30/11/2004              | L.DEPLANQUE             | Version 0.1ae :
*                          |                         | corection d'un bugg :
*                          |                         | Suppression de l'envoi
*                          |                         | d'un son dans le canal
*                          |                         | 0 pour simulation son
*                          |                         | cassette
* ------------------------------------------------------------------------------
*  02/06/2005              | L.DEPLANQUE             | Version 0.1ag :
*                          |                         | Gestion des fichiers .WAV
*                          |                         | pour la lecture depuis
*                          |                         | la cassette
* ------------------------------------------------------------------------------
*  06/12/2006              | L.DEPLANQUE             | Version 0.1am :
*                          |                         | Variable "VBL" remplace
*                          |                         | par "VSync" et dplace
*                          |                         | dans le module CRTC
* ------------------------------------------------------------------------------
*  13/05/2007              | L.DEPLANQUE             | Version 0.1an :
*                          |                         | Correction d'un petit bugg
*                          |                         | qui rinitialisait le
*                          |                         | canal sonore  chaque
*                          |                         | appel de la fonction
*                          |                         | SetSoundCas
* ------------------------------------------------------------------------------
********************************************************** !END! **************/


#include  <stdio.h>

#include  "Types.h"
#include  "Plateform.h"
#include  "AY8912.h"
#include  "Crtc.h"
#include  "Ppi.h"
#ifdef USE_LOG
#include  "Log.h"
#endif
#include  "Sound.h"
#include  "Config.h"


#define     MODULENAME      "PPI"


#define PRINTER_BUSY    0x40    // Printer busy

#define REFRESH_HZ      0x10    // Screen refresh = 50Hz

#define CONSTRUCTEUR    0x07    // valeurs possibles :
                                // 0x00 = Isp
                                // 0x01 = Triumph
                                // 0x02 = Saisho
                                // 0x03 = Solavox
                                // 0x04 = Awa
                                // 0x05 = Schneider
                                // 0x06 = Orion
                                // 0x07 = Amstrad


#ifdef USE_TAPE
#define SIZE_BUF_TAPE   0x800   // Taille buffer tampon cassette
                                // !!! Doit tre une puissance de 2 !!!
#endif

/********************************************************* !NAME! **************
* Nom : modePSG
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mode du PSG
*
********************************************************** !0! ****************/
static int modePSG;


/********************************************************* !NAME! **************
* Nom : RegPSGSel
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de registre du PSG slectionn
*
********************************************************** !0! ****************/
int RegPSGSel;


/********************************************************* !NAME! **************
* Nom : clav
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Matrice du clavier (16 * 8 bits)
*
********************************************************** !0! ****************/
int clav[ 16 ];


/********************************************************* !NAME! **************
* Nom : RegsPPI
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Registres inernes du PPI 8255
*
********************************************************** !0! ****************/
int RegsPPI[ 4 ];


/********************************************************* !NAME! **************
* Nom : Output
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Sorties du PPI 8255
*
********************************************************** !0! ****************/
static int Output[ 3 ];


/********************************************************* !NAME! **************
* Nom : Input
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Entres du PPI 8255
*
********************************************************** !0! ****************/
static int Input[ 3 ];


/********************************************************* !NAME! **************
* Nom : Mask
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Masques calculs pour masquage entres/sorties
*
********************************************************** !0! ****************/
static int Mask[ 3 ];


/********************************************************* !NAME! **************
* Nom : ligneClav
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de ligne du clavier slectionne
*
********************************************************** !0! ****************/
static int ligneClav;


#ifdef USE_TAPE
/********************************************************* !NAME! **************
* Nom : TpsTapeIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Nombre de cycles Z80  attendre avant appel routine lecture
*               cassette
*
********************************************************** !0! ****************/
int TpsTapeIn = 0;


/********************************************************* !NAME! **************
* Nom : TpsTapeOut
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Nombre de cycles Z80  attendre avant appel routine criture
*               cassette
*
********************************************************** !0! ****************/
int TpsTapeOut = 0;


/********************************************************* !NAME! **************
* Nom : fCasIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Handle fichier lecture cassette
*
********************************************************** !0! ****************/
static FILE * fCasIn = NULL;


/********************************************************* !NAME! **************
* Nom : fCasOut
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Handle fichier criture cassette
*
********************************************************** !0! ****************/
static FILE * fCasOut = NULL;


/********************************************************* !NAME! **************
* Nom : BitTapeIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Bit de lecture cassette pour le port F5xx
*
********************************************************** !0! ****************/
static UBYTE BitTapeIn = 0;


/********************************************************* !NAME! **************
* Nom : BufTape
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Buffer fichier cassette
*
********************************************************** !0! ****************/
static UBYTE BufTape[ SIZE_BUF_TAPE ];


/********************************************************* !NAME! **************
* Nom : PosBufTape
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Index sur buffer fichier cassette
*
********************************************************** !0! ****************/
static int PosBufTape;


/********************************************************* !NAME! **************
* Nom : FreqCpcIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Frquence du son jou lors de la lecture cassette
*
********************************************************** !0! ****************/
static int FreqCpcIn;


/********************************************************* !NAME! **************
* Nom : FreqCpcOut
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Frquence du son jou lors de l'criture cassette
*
********************************************************** !0! ****************/
static int FreqCpcOut;


/********************************************************* !NAME! **************
* Nom : NbOctetsOut
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Nombre d'octets cris dans le fichier Wav
*
********************************************************** !0! ****************/
static unsigned int NbOctetsOut;


/********************************************************* !NAME! **************
* Nom : MemoSoundCasOff 
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique que la sortie son cassette est 'Off'
*
********************************************************** !0! ****************/
static int MemoSoundCasOff = 0;


/********************************************************* !NAME! **************
* Nom : CloseTapOut
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Fermeture fichier cassette en criture
*
* Rsultat    : /
*
* Variables globales modifies : fCasOut
*
********************************************************** !0! ****************/
void CloseTapOut( void )
{
    StWave EnteteWaveOut;

    if ( fCasOut )
        {
        if ( PosBufTape )
            {
            fwrite( BufTape, PosBufTape, 1, fCasOut );
            NbOctetsOut += PosBufTape;
            }
        fseek( fCasOut, 0, SEEK_SET );
        fread( &EnteteWaveOut, sizeof( EnteteWaveOut), 1, fCasOut );
        fseek( fCasOut, 0, SEEK_SET );
        EnteteWaveOut.Size1 = NbOctetsOut + sizeof( EnteteWaveOut );
        EnteteWaveOut.Size2 = NbOctetsOut;
        fwrite( &EnteteWaveOut, sizeof( EnteteWaveOut), 1, fCasOut );
        fclose( fCasOut );
        fCasOut = NULL;
        }
    PosBufTape = 0;
    if ( UseSoundCas )
        {
        MemoSoundCasOff = 1;
        SetSound( 1, 0, 0 );
        SetSound( 4, 0, 0 );
        }
    TpsTapeOut = 0;
}


/********************************************************* !NAME! **************
* Nom : CloseTapIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Fermeture fichier cassette en lecture
*
* Rsultat    : /
*
* Variables globales modifies : fCasIn
*
********************************************************** !0! ****************/
void CloseTapIn( void )
{
    if ( fCasIn )
        {
        fclose( fCasIn );
        fCasIn = NULL;
        }
    if ( UseSoundCas )
        {
        MemoSoundCasOff = 1;
        SetSound( 1, 0, 0 );
        SetSound( 4, 0, 0 );
        }
    TpsTapeIn = 0;
}


/********************************************************* !NAME! **************
* Nom : OpenTapWrite
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ouverture fichier cassette en criture
*
* Rsultat    : /
*
* Variables globales modifies : fCas
*
********************************************************** !0! ****************/
void OpenTapWrite( char * Nom )
{
    StWave EnteteWaveOut;

    CloseTapOut();
    fCasOut = fopen( Nom, "w+b" );
    if ( fCasOut )
        {
        memset( &EnteteWaveOut, 0, sizeof( EnteteWaveOut ) );
        memcpy( EnteteWaveOut.Tag1, "RIFF", 4 );
        memcpy( EnteteWaveOut.Tag2, "WAVEfmt ", 8 );
        EnteteWaveOut.Tag2[ 8 ] = 0x10;
        EnteteWaveOut.Tag2[ 12 ] = 0x1;
        memcpy( EnteteWaveOut.Tag3, "data", 4 );
        EnteteWaveOut.Mode = 1;
        EnteteWaveOut.Format = 8;
        EnteteWaveOut.Freq = FreqWavOut;
        EnteteWaveOut.NbBytes = 1;
        EnteteWaveOut.BytePerSec = FreqWavOut;
        FreqCpcOut = FreqWavOut;
        TpsTapeOut = 1000000 / FreqCpcOut;
        fwrite( &EnteteWaveOut, sizeof( EnteteWaveOut ), 1, fCasOut );       
        NbOctetsOut = 0;
        }
}


/********************************************************* !NAME! **************
* Nom : OpenTapRead
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ouverture fichier cassette en lecture
*
* Rsultat    : /
*
* Variables globales modifies : fCas
*
********************************************************** !0! ****************/
void OpenTapRead( char * Nom )
{
    StWave Entete;

    CloseTapIn();
    fCasIn = fopen( Nom, "rb" );
    if ( fCasIn )
        {
        fread( &Entete, sizeof( Entete ), 1, fCasIn );
        if ( Entete.Mode == 1 && Entete.NbBytes == 1 && Entete.Format == 8 )
            {
            TpsTapeIn = 1000000 / Entete.Freq;
            FreqCpcIn = Entete.Freq;
            }
        else
            {
            // Format wav non reconnu
            Erreur( "Format de fichier .WAV non support." );
            CloseTapIn();
            }
        }
}


/********************************************************* !NAME! **************
* Nom : SetPosTapeIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Positionne le fichier cassette entre  une position prcise
*
* Rsultat    : /
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
void SetPosTapeIn( int Pos )
{
    if ( fCasIn && FreqCpcIn )
        fseek( fCasIn, sizeof( StWave ) + Pos * FreqCpcIn, SEEK_SET );
}


/********************************************************* !NAME! **************
* Nom : GetPosTapeIn
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Retourne la position du fichier cassette entre
*
* Rsultat    : La position convertie en secondes ou -1
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
int GetPosTapeIn( void )
{
    if ( fCasIn && FreqCpcIn )
        return( ( ftell( fCasIn ) - sizeof( StWave ) ) / FreqCpcIn );

    return( -1 );
}


/********************************************************* !NAME! **************
* Nom : SetSoundCas
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Arrte le son de la cassette si le moteur est OFF
*
* Rsultat    : /
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
static void SetSoundCas( void )
{
    if ( ! ( Output[ 2 ] & 0x10 ) && ! MemoSoundCasOff )
        {
        MemoSoundCasOff = 1;
        SetSound( 1, 0, 0 );
        SetSound( 4, 0, 0 );
        }
}


/********************************************************* !NAME! **************
* Nom : SoundCas
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Joue un son en fonction des bits lecture/criture cassette
*
* Rsultat    : /
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
static void SoundCas( int BitCas, int FreqCPC )
{
    static int ValBit = 0;
    static int CntBit = 0;
    static int CntFreq = 0;

    if ( BitCas != ValBit )
        {
        ValBit = BitCas;
        CntFreq += 32;
        }
    CntBit += 64;
    if ( CntBit >= FreqCPC )
        {
        MemoSoundCasOff = 0;
        SetSound( 1, CntFreq, 32 );
        SetSound( 4, CntFreq, 8 );
        CntBit = 0;
        CntFreq = 0;
        }
}


/********************************************************* !NAME! **************
* Nom : WriteCas
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ecriture d'un bit dans le fichier cassette
*
* Rsultat    : /
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
void WriteCas( void )
{
    int BitTapeOut = Output[ 2 ] & 0x20;
    if ( Output[ 2 ] & 0x10 )
        {
        if ( UseSoundCas )
            SoundCas( BitTapeOut, FreqCpcOut );

        BufTape[ PosBufTape++ ] = ( BYTE )( 0x40 + ( BitTapeOut << 2 ) );
        PosBufTape &= ( SIZE_BUF_TAPE - 1 );
        if ( ! PosBufTape )
            {
            NbOctetsOut += SIZE_BUF_TAPE;
            if ( fCasOut )
                fwrite( BufTape, SIZE_BUF_TAPE, 1, fCasOut );
            }
        }
}


/********************************************************* !NAME! **************
* Nom : ReadCas
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Lecture d'un bit depuis le fichier cassette
*
* Rsultat    : /
*
* Variables globales modifies : BitTapeIn
*
********************************************************** !0! ****************/
void ReadCas( void )
{
    BitTapeIn = 0;
    if ( Output[ 2 ] & 0x10 )
        {
        if ( ! PosBufTape && fCasIn )
            fread( BufTape, SIZE_BUF_TAPE, 1, fCasIn );

        if ( BufTape[ PosBufTape++ ] > 0x85 )
            BitTapeIn = 0x80;

        PosBufTape &= ( SIZE_BUF_TAPE - 1 );
        if ( UseSoundCas )
            SoundCas( BitTapeIn, FreqCpcIn );
        }
}
#endif


/********************************************************* !NAME! **************
* Nom : UpdatePSG
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mise  jour des registres du PSG en fonction du port A du PPI
*
* Rsultat    : /
*
* Variables globales modifies : RegPSGSel
*
********************************************************** !0! ****************/
static void UpdatePSG( void )
{
    switch ( modePSG )
        {
        case 2:
            Write8912( RegPSGSel, Output[ 0 ] );
            break;

        case 3:
            RegPSGSel = Output[ 0 ];
            break;
        }
}


/********************************************************* !NAME! **************
* Nom : PPI_WriteControl
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ecriture du registre de controle du PPI
*
* Rsultat    : /
*
* Variables globales modifies : RegsPPI, Output, Mask
*
********************************************************** !0! ****************/
static void PPI_WriteControl( int val )
{
    RegsPPI[ 3 ] = val;
    if ( val & 0x80 )
        {
        RegsPPI[ 0 ] = RegsPPI[ 2 ] = 0;
        Mask[ 0 ] = ( val & 0x10 ) ? 0xFF : 0;
        Mask[ 2 ] = 0xFF;
        if ( ! ( val & 0x01 ) )
            Mask[ 2 ] &= 0xF0;

        if ( ! ( val & 0x08 ) )
            Mask[ 2 ] &= 0x0F;
        }
    else
        {
        int BitMask = 1 << ( ( val >> 1 ) & 0x07 );
        if ( val & 1 )
            RegsPPI[ 2 ] |= BitMask;
        else
            RegsPPI[ 2 ] &= ~BitMask;
        }
    Output[ 0 ] = ( RegsPPI[ 0 ] & ~Mask[ 0 ] ) | Mask[ 0 ];
    Output[ 2 ] = ( RegsPPI[ 2 ] & ~Mask[ 2 ] ) | Mask[ 2 ];
#ifdef USE_TAPE
    if ( UseSoundCas )
        SetSoundCas();
#endif
}


/********************************************************* !NAME! **************
* Nom : WritePPI
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ecriture d'un registre du PPI
*
* Rsultat    : /
*
* Variables globales modifies : RegsPPI, Output, ligneClav, modePSG
*
********************************************************** !0! ****************/
void WritePPI( int adr, int val )
{
    static char Ligne[ 16 ];

    switch( ( adr >> 8 ) & 3 )
        {
        case 0 : // 0xF4xx
            RegsPPI[ 0 ] = val;
            Output[ 0 ] = ( val & ~Mask[ 0 ] ) | Mask[ 0 ];
            UpdatePSG();
            break;

#ifdef USE_LOG
        case 1 : // 0xF5xx
            Log( MODULENAME, "Tentative criture port PPI F500", LOG_INFO );
            break;
#endif
        case 2 : // 0xF6xx
            RegsPPI[ 2 ] = val;
            Output[ 2 ] = ( val & ~Mask[ 2 ] ) | Mask[ 2 ];
            ligneClav = Output[ 2 ] & 0x0F;
            modePSG = Output[ 2 ] >> 6;
            UpdatePSG();
#ifdef USE_TAPE
            if ( UseSoundCas )
                SetSoundCas();
#endif
            break;

        case 3 : // 0xF7xx
            PPI_WriteControl( val );
            break;
        }
}


/********************************************************* !NAME! **************
* Nom : ReadPPI
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Lecture d'un registre du PPI
*
* Rsultat    : Le registre du PPI selectionn
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
int ReadPPI( int adr )
{
    switch( ( adr >> 8 ) & 3 )
        {
        case 0 : // 0xF4xx
            if ( modePSG == 1 )
                {
                if ( RegPSGSel == 14 )
                    return( clav[ ligneClav ] );

                return( RegsPSG[ RegPSGSel ] );
                }
#ifdef USE_LOG
            Log( MODULENAME, "Lecture port F400 et modePsg <> 1", LOG_INFO );
#endif
            return( 0xFF );

        case 1 : // 0xF5xx
            return( ( CONSTRUCTEUR << 1 )
//                  | PRINTER_BUSY
                  | REFRESH_HZ
                  | VSync
#ifdef USE_TAPE
                  | BitTapeIn
#endif
                  ); // Port B toujours en lecture

        case 2 : // 0xF6xx
            return( ( Input[ 2 ] & Mask[ 2 ] ) 
                  | ( RegsPPI[ 2 ] & ~Mask[ 2 ] )
                  );
        }
    return( 0xFF );
}
