/******************************************************************************/
/* Configuration pour l'archivage des diffrents lments du fichier source   */
/******************************************************************************/
// !CONFIG!=/L/* /R/* /W"* Nom : "
// Dfinition du systme       !CONFIG!=/V1!EMULATEUR CPC!
// Dfinition du sous systme  !CONFIG!=/V2!PC-CPC!
// Dfinition du sous ensemble !CONFIG!=/V3!Accessoires!
// Dfinition du module        !CONFIG!=/V4!RomPack!
/******************************************************************************/

/********************************************************* !NAME! **************
* !./FLE!
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\Fichiers
********************************************************** !0! *****************
* ------------------------------------------------------------------------------
*          SYSTEME         |      SOUS SYSTEME       |      SOUS ENSEMBLE
* ------------------------------------------------------------------------------
*  EMULATEUR CPC           | PC-CPC                  | Accessoires
* -----------------------------------------------------------------------------
*  Fichier     : ROMPACK.C             | Version : 0.1an
* ------------------------------------------------------------------------------
*  Date        : 30/11/2005            | Auteur  : L.DEPLANQUE
* ------------------------------------------------------------------------------
*  Description : Gestion des copdes op. utilises par la rom pack
*
* ------------------------------------------------------------------------------
*  Historique  :
*           Date           |         Auteur          |       Description
* ------------------------------------------------------------------------------
*  30/11/2005              | L.DEPLANQUE             | creation
* ------------------------------------------------------------------------------
*  18/12/2005              | L.DEPLANQUE             | Version 0.1aj :
*                          |                         | Ajout instruction ED_09
* ------------------------------------------------------------------------------
*  16/02/2006              | L.DEPLANQUE             | Version 0.1ak :
*                          |                         | Ajout instruction ED_0A
* ------------------------------------------------------------------------------
*  27/06/2006              | L.DEPLANQUE             | Version 0.1al :
*                          |                         | Ajout instruction ED_0B
*                          |                         |  ED_16
* ------------------------------------------------------------------------------
*  20/09/2006              | L.DEPLANQUE             | Version 0.1am :
*                          |                         | Ajout instruction ED_17
*                          |                         |  ED_1C
* ------------------------------------------------------------------------------
*  22/03/2007              | L.DEPLANQUE             | Version 0.1an :
*                          |                         | Ajout instruction ED_1D
*                          |                         |  ED_1F
*                          |                         | Correction nombre de 
*                          |                         | cycles nops retourns
*                          |                         | par les instructions
*                          |                         | ED_00  ED_1C : 2 nops
*                          |                         | au lieu de 4
* ------------------------------------------------------------------------------
********************************************************** !END! **************/


#include  <windows.h>
#include  <stdio.h>

#include  "Types.h"
#include  "Z80.h"
#include  "Vga.h"
#include  "Pack.h"
#include  "Conversion.h"
#include  "Plateform.h"
#ifdef USE_SAVE_BMP
#include  "SauveBmp.h"
#endif
#include  "SnapShot.h"
#include  "WriteAvi.h"
#include  "WriteYM.h"
#ifdef USE_NEWDISC
#include  "NewDisc.h"
#endif
#include  "RomPack.h"
#include  "Config.h"
#ifdef USE_DESASM
#include  "Desasm.h"
#endif
#include  "Upd.h"
#ifdef USE_LOG
#include  "log.h"
#endif


#define     MODULENAME      "Rompack"


#ifdef USE_ROM_PACK

/********************************************************* !NAME! **************
* Nom : ConvLigne
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique si la conversion d'adresse cran doit se faire par
*               lignes ou par colonne
*
********************************************************** !0! ****************/
static BOOL ConvLigne = FALSE;


/********************************************************* !NAME! **************
* Nom : ConvCoord
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique si il y a lieu de faire une conversion des coordonnes
*               des fentres  compacter
*
********************************************************** !0! ****************/
static BOOL ConvCoord = FALSE;


/********************************************************* !NAME! **************
* Nom : SymbVidFile
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Pointeur de fichier vers vido symbos
*
********************************************************** !0! ****************/
static FILE * SymbVidFile = NULL;


/********************************************************* !NAME! **************
* Nom : SymbVidBuffer
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Buffer de lecture fichier vido symbos
*
********************************************************** !0! ****************/
static BYTE SymbVidBuffer[ 0x8000 ];


/********************************************************* !NAME! **************
* Nom : SymbVidSizeX
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Taille en X vido symbos
*
********************************************************** !0! ****************/
static int SymbVidSizeX;


/********************************************************* !NAME! **************
* Nom : SymbVidSizeY
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Taille en Y vido symbos
*
********************************************************** !0! ****************/
static int SymbVidSizeY;


/********************************************************* !NAME! **************
* Nom : SymbVidBufSize
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Taille du buffer frame vido symbos
*
********************************************************** !0! ****************/
static int SymbVidBufSize;


/********************************************************* !NAME! **************
* Nom : ED_00
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 00 (info PC-CPC)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_00( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_00 (InitRomPack)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "DE                   = 0x%04X", RegDE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "AF                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    RegA = 0xAA;
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_01
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 01 (PackScr)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_01( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_01 (PackScr)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "DE                   = 0x%04X", RegDE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "AF                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = PackScreen( RegHL, ConvLigne );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_02
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 02 (PackWin)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_02( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_02 (PackWin)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "B                    = 0x%04X", RegB );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "C                    = 0x%04X", RegC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "D                    = 0x%04X", RegD );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "E                    = 0x%04X", RegE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "AF                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = PackWindow( RegE, RegD, RegC, RegB, RegHL, ConvLigne, ConvCoord );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_03
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 03 (DepkScr)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_03( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_03 (DepkScr)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
//    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    DepackScreen( RegHL, ConvLigne );
//    WriteVGA( DecodeurAdresse );
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_04
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 01 (DepkWin)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_04( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_04 (DepkWin)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
//    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    DepackWindow( RegHL, ConvLigne );
//    WriteVGA( DecodeurAdresse );
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_05
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 05 (PackWinDC)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_05( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_05 (PackWinDC)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "B                    = 0x%04X", RegB );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "C                    = 0x%04X", RegC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "D                    = 0x%04X", RegD );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "E                    = 0x%04X", RegE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "AF                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = PackWinDC( RegHL, FALSE, ConvLigne, ConvCoord );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
    Poke8Ext( ( USHORT )( RegIX + 2 ), ( UBYTE )( ad >> 16 ) );
    Poke8Ext( ( USHORT )( RegIX + 3 ), ( UBYTE )( ad >> 24 ) );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_06
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 06 (PackBuff)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_06( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_06 (PackBuff)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "DE                   = 0x%04X", RegDE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = PackBuffer( RegHL, RegDE, RegBC );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_07
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 07 (DepkBuff)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_07( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_07 (DepkBuff)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "DE                   = 0x%04X", RegDE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
//    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = DepackBuffer( RegHL, RegDE );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
//    WriteVGA( DecodeurAdresse );
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_08
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 08 (PackWinDiff)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_08( void )
{
    int ad;

#ifdef USE_LOG
    Log( MODULENAME, "ED_08 (PackWinDiff)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "IX                   = 0x%04X", RegIX );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ad = PackWinDC( RegHL, TRUE, ConvLigne, ConvCoord );
    Poke8Ext( RegIX, ( UBYTE )ad );
    Poke8Ext( ( USHORT )( RegIX + 1 ), ( UBYTE )( ad >> 8 ) );
    Poke8Ext( ( USHORT )( RegIX + 2 ), ( UBYTE )( ad >> 16 ) );
    Poke8Ext( ( USHORT )( RegIX + 3 ), ( UBYTE )( ad >> 24 ) );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_09
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 09 (InitBuffDiff)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_09( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_09 (InitBuffDiff)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    InitBuffDiff( RegHL );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0A
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0A (ConvertBitmap)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0A( void )
{
    static char Nom[ 258 ];
    static char Argument[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_0A (ConvertBitmap)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "DE                   = 0x%04X", RegDE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    for ( i = 0; i < RegC; i++ )
        Argument[ i ] = Peek8Ext( ( USHORT )( RegDE + i ) );

    Argument[ i ] = 0;

//    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    ConvertBitmap( Nom, Argument );
//    WriteVGA( DecodeurAdresse );
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0B
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0B (ConvLigne)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0B( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_0B (ConvLigne)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    ConvLigne = RegA;
    ConvCoord = RegB;
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0C
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0C (SauveBitmap)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0C( void )
{
#ifdef USE_SAVE_BMP
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_0C (SauveBitmap)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    SauveBitmap( Nom );
//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0D
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0D (Turbo On/Off)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0D( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_0D (Turbo On/Off)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "AF                   = 0x%04X", RegAF );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    Turbo = RegA;
//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0E
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0E (ReadSnap)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0E( void )
{
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_0E (ReadSnap)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    LireSnap( Nom );
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_0F
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 0F (SaveSnap)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_0F( void )
{
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_0F (SaveSnap)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

//    ___C9();
    SauveSnap( Nom );
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_10
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 10 (Debug On)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_10( void )
{
#ifdef USE_DESASM
#ifdef USE_LOG
    Log( MODULENAME, "ED_10 (DebugOn)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    if ( ! SmallScreen )
        {
        DebugMode = 1;
        Cont = 1;
        }
//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_11
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 11 (SetDskFile)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_11( void )
{
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_11 (SetDskFile)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;


    if ( * Nom )
        {
#ifdef USE_LOG
        char Tmp[ 512 ];
        sprintf( Tmp, "Chargement disquette : %s", Nom );
        Log( MODULENAME, Tmp, LOG_INFO );
#endif
        strcpy( NomDisc, Nom );
#ifdef USE_NEWDISC
        NewDiscOn = FALSE;
#endif
        EjectDiskUPD( hWnd );
        SetDiskUPD( hWnd, Nom );
        }
    AfficheInfos();

//    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_12
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 12 (SaveAvi)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_12( void )
{
#ifdef USE_SAVE_AVI
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_12 (SaveAvi)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    if ( ! WriteAviStarted() && * Nom )
        WriteAviInit( hWnd
                    , Nom
                    , SmallScreen ? TAILLE_X_LOW : TAILLE_X_HIGH
                    , SmallScreen ? TAILLE_Y_LOW : TAILLE_Y_HIGH
                    );

//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_13
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 13 (EndAvi)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_13( void )
{
#ifdef USE_SAVE_AVI
#ifdef USE_LOG
    Log( MODULENAME, "ED_13 (EndAvi)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    if ( WriteAviStarted() )
        WriteAviEnd( hWnd );

//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_14
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 14 (SaveYM)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_14( void )
{
#ifdef USE_SAVE_YM
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_14 (SaveYM)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    if ( ! WriteYMStarted() && * Nom )
        WriteYMInit( hWnd, Nom );

//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_15
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 15 (EndYM)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_15( void )
{
#ifdef USE_SAVE_YM
#ifdef USE_LOG
    Log( MODULENAME, "ED_15 (EndYM)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    if ( WriteYMStarted() )
        WriteYMEnd( hWnd );

//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_16
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 16 (SetDirectDir)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_16( void )
{
#ifdef USE_NEWDISC
    static char Nom[ 258 ];
    int i;

#ifdef USE_LOG
    Log( MODULENAME, "ED_16 (SetDirectDir)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    if ( * Nom )
        {
        strcpy( DirEmuDisc, Nom );
        FlushCloseIn();
        FlushCloseOut();
        NewDiscOn = TRUE;
        AfficheInfos();
        }
//    ___C9();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_17
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 17 (SymbVidOpen)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_17( void )
{
    static char Nom[ 258 ];
    int i;

    FLAGS |= FLAG_Z;
    if ( SymbVidFile )
        ED_19();

#ifdef USE_LOG
    Log( MODULENAME, "ED_17 (SymbVidOpen)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "BC                   = 0x%04X", RegBC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    for ( i = 0; i < RegB; i++ )
        Nom[ i ] = Peek8Ext( ( USHORT )( RegHL + i ) );

    Nom[ i ] = 0;

    if ( * Nom )
        {
        SymbVidFile = fopen( Nom, "rb" );
        if ( SymbVidFile )
            {
            FLAGS &= ~FLAG_Z;
            fread( SymbVidBuffer, 0x200, 1, SymbVidFile );
            SymbVidSizeX = ( SymbVidBuffer[ 0x0E ] + ( SymbVidBuffer[ 0x0F ] << 8 ) ) >> 2;
            SymbVidSizeY = SymbVidBuffer[ 0x10 ] + ( SymbVidBuffer[ 0x11 ] << 8 );
            SymbVidBufSize = SymbVidBuffer[ 0x33 ] + ( SymbVidBuffer[ 0x34 ] << 8 );
            }
        }
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_18
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 18 (SymbVidRead)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_18( void )
{
    FLAGS |= FLAG_Z;
    if ( SymbVidFile )
        {
#ifdef USE_LOG
        Log( MODULENAME, "ED_18 (SymbVidRead)", LOG_DEBUG );
        sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
        sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
        sprintf( MsgLog , "HL                   = 0x%04X", RegHL );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
        int Size = min( sizeof( SymbVidBuffer ), SymbVidBufSize );
        RegA = ( BYTE )fread( SymbVidBuffer, Size, 1, SymbVidFile );
        MemHostToCpc( SymbVidBuffer, RegHL, Size );
        FLAGS &= ~FLAG_Z;
        }
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_19
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 19 (SymbVidPlay)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_19( void )
{
    FLAGS |= FLAG_Z;
    if ( SymbVidFile )
        {
#ifdef USE_LOG
        Log( MODULENAME, "ED_19 (SymbVidPlay)", LOG_DEBUG );
        sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
        sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
        int Size = min( sizeof( SymbVidBuffer ), SymbVidBufSize );
        RegA = ( BYTE )fread( SymbVidBuffer, Size, 1, SymbVidFile );
        for ( int l = 0; l < SymbVidSizeY; l++ )
            {
            int AdrCPC = ( l >> 3 ) * 80 + ( l & 7 ) * 0x800;
            MemHostToCpc( &SymbVidBuffer[ l * SymbVidSizeX ]
                        , ( USHORT )( 0xC000 + AdrCPC )
                        , SymbVidSizeX
                        );
            }
        
        FLAGS &= ~FLAG_Z;
        }
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1A
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1A (SymbVidClose)
*
* Rsultat    : 4
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1A( void )
{
    FLAGS |= FLAG_Z;
    if ( SymbVidFile )
        {
#ifdef USE_LOG
        Log( MODULENAME, "ED_1A (SymbVidClose)", LOG_DEBUG );
        sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
        sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
        Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
        FLAGS &= ~FLAG_Z;
        fclose( SymbVidFile );
        SymbVidFile = NULL;
        }
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1B
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1B (ProfileStart)
*
* Rsultat    : 2
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1B( void )
{
#ifdef USE_PROFILE
#ifdef USE_LOG
    Log( MODULENAME, "ED_1B (ProfileStart)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    StartProfile();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1C
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1C (ProfileStop)
*
* Rsultat    : 2
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1C( void )
{
#ifdef USE_PROFILE
#ifdef USE_LOG
    Log( MODULENAME, "ED_1C (ProfileStop)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    EndProfile();
#endif
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1D
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1D (SetNewDisc)
*
* Rsultat    : 2
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1D( void )
{
#ifdef USE_LOG
    Log( MODULENAME, "ED_1D (SetNewDisc)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    FlushCloseIn();
    FlushCloseOut();
    NewDiscOn = RegA;
    AfficheInfos();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1E
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1E (DrawCadreWin)
*
* Rsultat    : 2
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1E( void )
{
    static BYTE BufMem[ 0x4000 ];

#ifdef USE_LOG
    Log( MODULENAME, "ED_1E (DrawCadreWin)", LOG_DEBUG );
    sprintf( MsgLog , "DecodeurAdresse      = 0x%02X", DecodeurAdresse );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "PC                   = 0x%04X", RegPC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "B                    = 0x%04X", RegB );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "C                    = 0x%04X", RegC );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "D                    = 0x%04X", RegD );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
    sprintf( MsgLog , "E                    = 0x%04X", RegE );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif

    WriteVGA( DecodeurAdresse | ROMSUP_OFF );
    MemCpcToHost( 0xC000, BufMem, 0x4000 );

    // Lignes horizontales
    for ( int c = RegB; c <= RegD; c++ )
        {
        BufMem[ c + ( RegC >> 3 ) * 80 + ( RegC & 7 ) * 0x800 ] ^= 0xFF;
        if ( RegC != RegE )
            BufMem[ c + ( RegE >> 3 ) * 80 + ( RegE & 7 ) * 0x800 ] ^= 0xFF;
        }

    // Lignes verticales
    for ( int l = RegC + 1; l < RegE; l++ )
        {
        int adr = ( l >> 3 ) * 80 + ( l & 7 ) * 0x800;
        BufMem[ RegB + adr] ^= 0xFF;
        if ( RegB != RegD )
            BufMem[ RegD + adr ] ^= 0xFF;
        }

    MemHostToCpc( BufMem, 0xC000, 0x4000 );
    WriteVGA( DecodeurAdresse );
    ___C9();
    return( 2 );
}


/********************************************************* !NAME! **************
* Nom : ED_1F
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Emulation spciale code op. ED 1F (Rserve)
*
* Rsultat    : 2
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
int ED_1F( void )
{
    return( 2 );
}


#endif
