/******************************************************************************/
/* Configuration pour l'archivage des diffrents lments du fichier source   */
/******************************************************************************/
// !CONFIG!=/L/* /R/* /W"* Nom : "
// Dfinition du systme       !CONFIG!=/V1!EMULATEUR CPC!
// Dfinition du sous systme  !CONFIG!=/V2!PC-CPC!
// Dfinition du sous ensemble !CONFIG!=/V3!Chips!
// Dfinition du module        !CONFIG!=/V4!VGA!
/******************************************************************************/

/********************************************************* !NAME! **************
* !./FLE!
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\Fichiers
********************************************************** !0! *****************
* ------------------------------------------------------------------------------
*          SYSTEME         |      SOUS SYSTEME       |      SOUS ENSEMBLE
* ------------------------------------------------------------------------------
*  EMULATEUR CPC           | PC-CPC                  | Chips
* ------------------------------------------------------------------------------
*  Fichier     : VGA.C                 | Version : 0.1an
* ------------------------------------------------------------------------------
*  Date        : 05/11/2002            | Auteur  : L.DEPLANQUE
* ------------------------------------------------------------------------------
*  Description : Emulation du circuit VGA
*
* ------------------------------------------------------------------------------
*  Historique  :
*           Date           |         Auteur          |       Description
* ------------------------------------------------------------------------------
*  05/11/2002              | L.DEPLANQUE             | creation
* ------------------------------------------------------------------------------
*  03/08/2004              | L.DEPLANQUE             | Version 0.1ab :
*                          |                         | Correction du mapping
*                          |                         | mmoire lors de la
*                          |                         | slection de la valeur
*                          |                         | 0xC3 sur le Gate array.
*                          |                         | Gestion de 16 roms
*                          |                         | d'extensions.
* ------------------------------------------------------------------------------
*  06/12/2004              | L.DEPLANQUE             | Version 0.1ae :
*                          |                         | Ajout de la rom
*                          |                         | d'extension de gestion
*                          |                         | compactage/dcompactage
* ------------------------------------------------------------------------------
*  19/10/2005              | L.DEPLANQUE             | Version 0.1ah:
*                          |                         | rom de gestion compactage/
*                          |                         | dcompactage utilise en
*                          |                         | tant que rom externe
* ------------------------------------------------------------------------------
*  08/12/2005              | L.DEPLANQUE             | Version 0.1ai:
*                          |                         | Modification paramtres
*                          |                         | couleurs pour sauvegarde
*                          |                         | BMP
* ------------------------------------------------------------------------------
*  16/12/2005              | L.DEPLANQUE             | Version 0.1aj:
*                          |                         | Prise en compte des
*                          |                         | en-ttes Amsdos dans les
*                          |                         | roms externes
* ------------------------------------------------------------------------------
*  06/10/2006              | L.DEPLANQUE             | Version 0.1am:
*                          |                         | Correction d'un bug
*                          |                         | dans la gestion des roms
*                          |                         | externes (pb multiface).
*                          |                         | Gestion 512K de ram.
*                          |                         | Adoucissement couleurs
* ------------------------------------------------------------------------------
*  22/03/2007              | L.DEPLANQUE             | Version 0.1an:
*                          |                         | Mmorisation nouveau
*                          |                         | mode demand avant
*                          |                         | affectation  la
*                          |                         | prochaine HBL
* ------------------------------------------------------------------------------
********************************************************** !END! **************/


#include  <memory.h>
#include  <stdio.h>

#include  "types.h"
#ifdef USE_MULTIFACE
#include  "multiface.h"
#endif
#ifdef USE_NEWDISC
#include  "NewDisc.h"
#endif
#include  "Plateform.h"
#include  "config.h"
#ifdef USE_LOG
#include  "log.h"
#endif
#include  "z80.h"
#include  "vga.h"
#include  "crtc.h"


#define     MODULENAME      "VGA"


/********************************************************* !NAME! **************
* Nom : MemCPC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mmoire du CPC
*
********************************************************** !0! ****************/
UBYTE MemCPC[ 0x90000 ];


/********************************************************* !NAME! **************
* Nom : TabPOKE
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mapping des critures vers mmoire CPC
*
********************************************************** !0! ****************/
UBYTE * TabPOKE[ 4 ];


/********************************************************* !NAME! **************
* Nom : TabPEEK
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mapping des lectures depuis mmoire CPC
*
********************************************************** !0! ****************/
UBYTE * TabPEEK[ 4 ];


/********************************************************* !NAME! **************
* Nom : TabCoul
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Couleurs R,V,B
*
********************************************************** !0! ****************/
int TabCoul[ 32 ];


/********************************************************* !NAME! **************
* Nom : TabInk
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de couleur
*
********************************************************** !0! ****************/
int TabInk[ 32 ];


/********************************************************* !NAME! **************
* Nom : RamSelect
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de ram slectionne (128K)
*
********************************************************** !0! ****************/
int RamSelect;


/********************************************************* !NAME! **************
* Nom : Bloc
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro du bloc de ram slectionne (512K)
*
********************************************************** !0! ****************/
static int Bloc;


/********************************************************* !NAME! **************
* Nom : MemoMode
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Nouveau mode cran slectionn
*
********************************************************** !0! ****************/
int MemoMode;


/********************************************************* !NAME! **************
* Nom : DecodeurAdresse
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : ROMs actives/dsactives
*
********************************************************** !0! ****************/
int DecodeurAdresse;


/********************************************************* !NAME! **************
* Nom : NumRomExt
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de rom suprieure slectionne
*
********************************************************** !0! ****************/
int NumRomExt;


/********************************************************* !NAME! **************
* Nom : PenSelect
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de stylot slectionn
*
********************************************************** !0! ****************/
int PenSelect;


/********************************************************* !NAME! **************
* Nom : ROMINF
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Rom infrieure
*
********************************************************** !0! ****************/
static UBYTE ROMINF[ 0x4000 ];


/********************************************************* !NAME! **************
* Nom : ROMEXT
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Roms d'extensions : 00 = RomSup, 07 = Rom disque (Amsdos)
*
********************************************************** !0! ****************/
static UBYTE ROMEXT[ MAX_ROM_EXT ][ 0x4000 ];


/********************************************************* !NAME! **************
* Nom : AdjRam
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Configuration de la ram en fonction du registre de slection
*
********************************************************** !0! ****************/
static int AdjRam[ 8 ][ 4 ][ 8 ] =
    {
    // C0       C1       C2       C3       C4       C5       C6       C7
    0x00000, 0x00000, 0x10000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x14000, 0x0C000, 0x10000, 0x14000, 0x18000, 0x1C000,
    0x08000, 0x08000, 0x18000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x1C000, 0x1C000, 0x1C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // C8       C9       CA       CB       CC       CD       CE       CF
    0x00000, 0x00000, 0x20000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x24000, 0x0C000, 0x20000, 0x24000, 0x28000, 0x2C000,
    0x08000, 0x08000, 0x28000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x2C000, 0x2C000, 0x2C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // D0       D1       D2       D3       D4       D5       D6       D7
    0x00000, 0x00000, 0x30000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x34000, 0x0C000, 0x30000, 0x34000, 0x38000, 0x3C000,
    0x08000, 0x08000, 0x38000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x3C000, 0x3C000, 0x3C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // D8       D9       DA       DB       DC       DD       DE       DF
    0x00000, 0x00000, 0x40000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x44000, 0x0C000, 0x40000, 0x44000, 0x48000, 0x4C000,
    0x08000, 0x08000, 0x48000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x4C000, 0x4C000, 0x4C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // E0       E1       E2       E3       E4       E5       E6       E7
    0x00000, 0x00000, 0x50000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x54000, 0x0C000, 0x50000, 0x54000, 0x58000, 0x5C000,
    0x08000, 0x08000, 0x58000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x5C000, 0x5C000, 0x5C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // E8       E9       EA       EB       EC       ED       EE       EF
    0x00000, 0x00000, 0x60000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x64000, 0x0C000, 0x60000, 0x64000, 0x68000, 0x6C000,
    0x08000, 0x08000, 0x68000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x6C000, 0x6C000, 0x6C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // F0       F1       F2       F3       F4       F5       F6       F7
    0x00000, 0x00000, 0x70000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x74000, 0x0C000, 0x70000, 0x74000, 0x78000, 0x7C000,
    0x08000, 0x08000, 0x78000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x7C000, 0x7C000, 0x7C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // F8       F9       FA       FB       FC       FD       FE       FF
    0x00000, 0x00000, 0x80000, 0x00000, 0x00000, 0x00000, 0x00000, 0x00000,
    0x04000, 0x04000, 0x84000, 0x0C000, 0x80000, 0x84000, 0x88000, 0x8C000,
    0x08000, 0x08000, 0x88000, 0x08000, 0x08000, 0x08000, 0x08000, 0x08000,
    0x0C000, 0x8C000, 0x8C000, 0x8C000, 0x0C000, 0x0C000, 0x0C000, 0x0C000,

    // 0123     0127     4567     0327     0423     0523     0623     0723
    };


/********************************************************* !NAME! **************
* Nom : SetMemCPC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Mapping de la mmoire du CPC en fonction de la slection des
*               roms et rams
*
* Rsultat    : /
*
* Variables globales modifies : TabPOKE, TabPEEK
*
********************************************************** !0! ****************/
static void SetMemCPC( void )
{
#ifdef USE_MULTIFACE
    TabPOKE[ 0 ] = MultifaceEnable && ( ! ( DecodeurAdresse & ROMINF_OFF ) ) ?
                       MULTIFACE_ROM :
                       &MemCPC[ AdjRam[ Bloc ][ 0 ][ RamSelect ] ];
    TabPEEK[ 0 ] = ( DecodeurAdresse & ROMINF_OFF ) ?
                       &MemCPC[ AdjRam[ Bloc ][ 0 ][ RamSelect ] ] :
                       MultifaceEnable ?
                           MULTIFACE_ROM :
                           ROMINF;
#else
    TabPOKE[ 0 ] = &MemCPC[ AdjRam[ Bloc ][ 0 ][ RamSelect ] ];
    TabPEEK[ 0 ] = ( DecodeurAdresse & ROMINF_OFF ) ?
                       &MemCPC[ AdjRam[ Bloc ][ 0 ][ RamSelect ] ] :
                       ROMINF;
#endif
    TabPOKE[ 1 ] =
    TabPEEK[ 1 ] = &MemCPC[ AdjRam[ Bloc ][ 1 ][ RamSelect ] ];

    TabPOKE[ 2 ] =
    TabPEEK[ 2 ] = &MemCPC[ AdjRam[ Bloc ][ 2 ][ RamSelect ] ];

    TabPOKE[ 3 ] = &MemCPC[ AdjRam[ Bloc ][ 3 ][ RamSelect ] ];

    TabPEEK[ 3 ] = ( DecodeurAdresse & ROMSUP_OFF ) ?
                   &MemCPC[ AdjRam[ Bloc ][ 3 ][ RamSelect ] ] :
                   ROMEXT[ NumRomExt ];

}


/********************************************************* !NAME! **************
* Nom : CheckAmsdos
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Vrifie si le buffer pass en paramtre correspond
*                une en-tte Amsdos
*
* Rsultat    : TRUE si en-tte Amsdos, FALSE sinon
*
* Variables globales modifies : /
*
********************************************************** !0! ****************/
static BOOL CheckAmsdos( UBYTE * Buf )
{
    int i, Checksum = 0;
    BOOL ModeAmsdos = FALSE;
    StAmsdos * pEntete = ( StAmsdos * )Buf;

    for ( i = 0; i < 67; i++ )
        Checksum += Buf[ i ];

    if ( ( pEntete->CheckSum == ( USHORT )Checksum ) && Checksum )
        ModeAmsdos = TRUE;

    return( ModeAmsdos );
}


/********************************************************* !NAME! **************
* Nom : WriteVGA
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Ecriture d'un registre du VGA
*
* Rsultat    : /
*
* Variables globales modifies : PenSelect, TabCoul, TabInk, DecodeurAdresse,
*                                MemoMode, CntHSync, IRQ, RamSelect
*
********************************************************** !0! ****************/
void WriteVGA( int val )
{
    static int RgbCPC32[ 32 ] =
        {
        0x666666,                 // Blanc            (13) -> #40
        0x666666,                 // Blanc            (13) -> #41
        0x00FE66,                 // Vert Marin       (19) -> #42
        0xFEFE66,                 // Jaune Pastel     (25) -> #43
        0x000066,                 // Bleu              (1) -> #44
        0xFE0066,                 // Pourpre           (7) -> #45
        0x006666,                 // Turquoise        (10) -> #46
        0xFE6666,                 // Rose             (16) -> #47
        0xFE0066,                 // Pourpre           (7) -> #48
        0xFEFE00,                 // Jaune vif        (24) -> #49
        0xFEFE00,                 // Jaune vif        (24) -> #4A
        0xFEFEFE,                 // Blanc Brillant   (26) -> #4B
        0xFE0000,                 // Rouge vif         (6) -> #4C
        0xFE00FE,                 // Magenta vif       (8) -> #4D
        0xFE6600,                 // Orange           (15) -> #4E
        0xFE66FE,                 // Magenta pastel   (17) -> #4F
        0x000066,                 // Bleu              (1) -> #50
        0x00FE66,                 // Vert Marin       (19) -> #51
        0x00FE00,                 // Vert vif         (18) -> #52
        0x00FEFE,                 // Turquoise vif    (20) -> #53
        0x000000,                 // Noir              (0) -> #54
        0x0000FE,                 // Bleu vif          (2) -> #55
        0x006600,                 // Vert              (9) -> #56
        0x0066FE,                 // Bleu ciel        (11) -> #57
        0x660066,                 // Magenta           (4) -> #58
        0x66FE66,                 // Vert pastel      (22) -> #59
        0x66FE00,                 // Vert citron      (21) -> #5A
        0x66FEFE,                 // Turquoise pastel (23) -> #5B
        0x660000,                 // Rouge             (3) -> #5C
        0x6600FE,                 // Mauve             (5) -> #5D
        0x666600,                 // Jaune            (12) -> #5E
        0x6666FE                  // Bleu pastel      (14) -> #5F
        };

    static int RgbCPC32Mono[ 32 ] =
        {
        0x7F7F7F,    // 13
        0x7F7F7F,    // 13
        0xBABABA,    // 19
        0xF5F5F5,    // 25
        0x090909,    // 1
        0x444444,    // 7
        0x626262,    // 10
        0x9C9C9C,    // 16
        0x444444,    // 7
        0xEBEBEB,    // 24
        0xEBEBEB,    // 24
        0xFFFFFF,    // 26
        0x3A3A3A,    // 6
        0x4E4E4E,    // 8
        0x939393,    // 15
        0xA6A6A6,    // 17
        0x090909,    // 1
        0xBABABA,    // 19
        0xB0B0B0,    // 18
        0xC4C4C4,    // 20
        0x000000,    // 0
        0x131313,    // 2
        0x585858,    // 9
        0x6B6B6B,    // 11
        0x272727,    // 4
        0xD7D7D7,    // 22
        0xCDCDCD,    // 21
        0xE1E1E1,    // 23
        0x1D1D1D,    // 3
        0x313131,    // 5
        0x757575,    // 12
        0x898989     // 14
        };

    int newVal = val & 0x1F;

    switch( val >> 6 )
        {
        case 0 :
            PenSelect = newVal;
            break;

        case 1 :
            switch( Monochrome )
                {
                case 0 :
                    TabCoul[ PenSelect ] = RgbCPC32[ newVal ];
                    break;
                case 1 :
                    TabCoul[ PenSelect ] = RgbCPC32Mono[ newVal ];
                    break;
                case 2 :
                    TabCoul[ PenSelect ] = RgbCPC32Mono[ newVal ] & 0x00FF00;
                    break;
                case 3 :
                    TabCoul[ PenSelect ] = RgbCPC32Mono[ newVal ] & 0xFFFF00;
                    break;
                }
            TabInk[ PenSelect ] = newVal;
            break;

        case 2 :
            DecodeurAdresse = val;
            MemoMode = val & 3;
            SetMemCPC();
            if ( val & 16 )
                {
                CntHSync = 0;
                IRQ = 0;
                }
            break;

        case 3 :
            RamSelect = val & 7;
            Bloc = Use512Ko * ( ( val >> 3 ) & 7 );
            SetMemCPC();
            break;
        }
}


/********************************************************* !NAME! **************
* Nom : WriteROM
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Slection du numro de rom suprieure
*
* Rsultat    : /
*
* Variables globales modifies : NumRomExt
*
********************************************************** !0! ****************/
void WriteROM( int val )
{
#ifdef USE_LOG
    sprintf( MsgLog, "Slection rom externe %d", val );
    Log( MODULENAME, MsgLog, LOG_DEBUG );
#endif
    NumRomExt = val;
    SetMemCPC();
}


/********************************************************* !NAME! **************
* Nom : InitMemCPC
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Initialisation : lecture des roms du cpc
*
* Rsultat    : TRUE si ok, FALSE sinon
*
* Variables globales modifies : ROMINF, ROMSUP, ROMDISC
*
********************************************************** !0! ****************/
BOOL InitMemCPC( void )
{
    static UBYTE RomLoc[ 0x4080 ];
    FILE * fp;
    BOOL ErreurExt = FALSE;
    int i;

    fp = fopen( LocRomInf, "rb" );
    if ( fp )
        {
        fread( ROMINF, sizeof( ROMINF ), 1, fp );
        fclose( fp );
        for ( i = 0; i < MAX_ROM_EXT; i++ )
            {
            if ( LocRomExt[ i ][ 0 ] )
                {
                fp = fopen( LocRomExt[ i ], "rb" );
                if ( fp )
                    {
                    fread( RomLoc, sizeof( RomLoc ), 1, fp );
                    if ( CheckAmsdos( RomLoc ) )
                        memcpy( RomLoc, &RomLoc[ 0x80 ], 0x4000 );

                    memcpy( ROMEXT[ i ], RomLoc, sizeof( ROMEXT[ i ] ) );
                    fclose( fp );
                    }
                else
                    {
                    ErreurExt = TRUE;
                    break;
                    }
                }
            else
                memcpy( ROMEXT[ i ], ROMEXT[ 0 ], sizeof( ROMEXT[ 0 ] ) );
            }

        if ( ! ErreurExt )
            {
            WriteVGA( 0x89 );
            WriteVGA( 0xC0 );
            return( TRUE );
            }
#ifdef USE_LOG
        else
            {
            sprintf( MsgLog, "Fichier %s non trouv.", LocRomExt[ i ] );
            Log( MODULENAME, MsgLog, LOG_WARNING );
            }
#endif
        }
#ifdef USE_LOG
    else
        {
        sprintf( MsgLog, "Fichier %s non trouv.", LocRomInf );
        Log( MODULENAME, MsgLog, LOG_WARNING );
        }
#endif
    return( FALSE );
}
