/******************************************************************************/
/* Configuration pour l'archivage des diffrents lments du fichier source   */
/******************************************************************************/
// !CONFIG!=/L/* /R/* /W"* Nom : "
// Dfinition du systme       !CONFIG!=/V1!EMULATEUR CPC!
// Dfinition du sous systme  !CONFIG!=/V2!PC-CPC!
// Dfinition du sous ensemble !CONFIG!=/V3!Accessoires!
// Dfinition du module        !CONFIG!=/V4!WriteYM!
/******************************************************************************/

/********************************************************* !NAME! **************
* !./FLE!
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\Fichiers
********************************************************** !0! *****************
* ------------------------------------------------------------------------------
*          SYSTEME         |      SOUS SYSTEME       |      SOUS ENSEMBLE
* ------------------------------------------------------------------------------
*  EMULATEUR CPC           | PC-CPC                  | Accessoires
* ------------------------------------------------------------------------------
*  Fichier     : WRITEYM.C             | Version : 0.1al
* ------------------------------------------------------------------------------
*  Date        : 20/07/2006            | Auteur  : L.DEPLANQUE
* ------------------------------------------------------------------------------
*  Description : Ecriture d'un fichier son au format YM (5)
*
* ------------------------------------------------------------------------------
*  Historique  :
*           Date           |         Auteur          |       Description
* ------------------------------------------------------------------------------
*  20/07/2006              | L.DEPLANQUE             | creation
* ------------------------------------------------------------------------------
********************************************************** !END! **************/


#include  <windows.h>
#include  <stdio.h>

#include  "Log.h"
#include  "Types.h"
#include  "AY8912.h"
#include  "WriteYM.h"
#include  "Plateform.h"


#ifdef USE_SAVE_YM

#define     MAX_YM_TRAMES       0x10000     // 22 minutes max.


/********************************************************* !NAME! **************
* Nom : YMEntrelace
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique si les Trames YM doivent tre entrelaces ou non
*
********************************************************** !0! ****************/
static BYTE YMEntrelace = 0;


/********************************************************* !NAME! **************
* Nom : NumTrame
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Numro de trame en cours d'criture
*
********************************************************** !0! ****************/
static int NumTrame = 0;


static BYTE TabTrameReg[ MAX_YM_TRAMES ][ 16 ];


/********************************************************* !NAME! **************
* Nom : YMFileStarted
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique si l'criture est dmarre ou non
*
********************************************************** !0! ****************/
static BOOL YMFileStarted = FALSE;


/********************************************************* !NAME! **************
* Nom : FirstWrite
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Indique premire criture
*
********************************************************** !0! ****************/
static BOOL FirstWrite;


/********************************************************* !NAME! **************
* Nom : FileYM
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Variables Globales
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\/FLE!, ligne : !./LN!
*
* Description : Handle du fichier YM
*
********************************************************** !0! ****************/
static FILE * FileYM = NULL;


/********************************************************* !NAME! **************
* Nom : WriteYMInit
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Initialisation (cration) du fichier YM
*
* Rsultat    : TRUE si ok, FALSE sinon
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
BOOL WriteYMInit( HWND hWnd, char * Nom )
{
    FileYM = fopen( Nom, "wb+ " );
    if ( FileYM )
        {
        YMFileStarted = TRUE;
        NumTrame = 0;
        FirstWrite = TRUE;
        return( TRUE );
        }
    else
        {
        // Impossible de crer le fichier ?
        wsprintf( MsgLog, "Impossible de crer le fichier %s", Nom );
        Erreur( MsgLog );
        }
    WriteYMEnd( hWnd );
    return( FALSE );
}


/********************************************************* !NAME! **************
* Nom : WriteYMFrame
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Ecrit une Frame (Registres AY) dans le fichier YM
*
* Rsultat    : TRUE si ok, FALSE sinon
*
* Variables globales modifies : NumTrame
*
********************************************************** !0! ****************/
BOOL WriteYMFrame( void )
{
    static int LastRegsPSG[ 16 ];

    BOOL Diff = ( NumTrame > 0 );

    if ( FirstWrite )
        {
        memcpy( LastRegsPSG, RegsPSG, sizeof( LastRegsPSG ) );
        FirstWrite = FALSE;
        }

    if ( WriteYMStarted() )
        {
        if ( NumTrame < MAX_YM_TRAMES )
            {
            if ( ! Diff )
                {
                for ( int i = 0; i < 14; i++ )
                    if ( LastRegsPSG[ i ] != RegsPSG[ i ] )
                        {
                        LastRegsPSG[ i ] = RegsPSG[ i ];
                        Diff = TRUE;
                        }
                }
            if ( Diff )
                {
                for ( int i = 0; i < 13; i++ )
                    TabTrameReg[ NumTrame ][ i ] = ( BYTE )RegsPSG[ i ];

                // Si le registre 13 n'a pas chang, on crit 0xFF
                TabTrameReg[ NumTrame ][ 13 ] = ( BYTE )( IsWritePSGReg13() ?
                                                            RegsPSG[ 13 ] :
                                                            0xFF 
                                                        );
                NumTrame++;
                }
            }
        else
            WriteYMEnd( hWnd );

        return( TRUE );
        }
    return( FALSE );
}


/**
 *
 * Process AYC crunching :
 * @param intBuffSize int defining the Compression Buffer Size (either 0x100 or 0x400)
 * @return Crunched array of Bytes
 */
static BYTE* doAYCCrunch( BYTE r, USHORT intRawLength, USHORT intBuffSize, USHORT* intCrunLen)
{
    // AYC crunching is as follow :
    // - A sequence is
    //        + A character
    //        + A lenght + offset
    // - First Byte is a character
    // - Flags are inputed every 8 sequences and bits represents the following sequences, ie : b00000111 means:
    //        + 5 characters
    //        + 3 previous sequences

    static BYTE arrCrunchedData[ 0x10000 ];
    USHORT intCrunchedLength = 0;

    /**
     * Minimum interresting size for a previous Pattern
     * For BuffSize = 0x100, it's interresting only if the previous matching pattern size is > 1
     * offset + length + 1 bit = 17 bits > 1 char + 1 bit = 9 bits
     * offset + length + 1 bit = 17 bits < 2 chars + 2 bits = 18 bits
     * For BuffSize = 0x400, it's interresting only if the previous matching pattern size is > 2
     * offset + length + 1 bit = 25 bits > 1 char + 1 bit = 9 bits
     * offset + length + 1 bit = 25 bits > 2 chars + 2 bits = 18 bits
     * offset + length + 1 bit = 25 bits < 3 chars + 3 bits = 27 bits
     */ 
    int intMinSeqSize = 1;
    if (intBuffSize > 0x100) intMinSeqSize++;

    // First Byte
    arrCrunchedData[intCrunchedLength++] = TabTrameReg[0][r];

    // Reserve flag place 
    int intOffsetFlag = intCrunchedLength++;

    // Main Crunching Loop
    int curr_index = 1;
    BYTE bsFlag = 0x00;
    int intSeqCounter = 8;
    
    while(curr_index<intRawLength)
    {
        // Put flag if needed
        if (intSeqCounter <= 0)
        {
            // Reset number of sequences
            intSeqCounter = 8;

            // Insert Flag in flow
            arrCrunchedData[intOffsetFlag] = bsFlag;

            // Reserve flag place 
            intOffsetFlag = intCrunchedLength++;

            // Clear Flag
            bsFlag = 0x00;
        }

        /********************************************************
         * Fast Crunching method                                *
         * We test until a byte isn't matching the buffered one *
         ********************************************************/

        // get Min index in buffer
        int index_min = curr_index - intBuffSize;
        index_min = (index_min<0)?0:index_min;
        int index_buffer = curr_index - 1;

        // get Max index in buffer + 1
        int index_max = curr_index + 0x100;
        index_max = (index_max>intRawLength)?intRawLength:index_max;

        // Set previous matching sequence size and index
        int intSeqSize = 0;
        int intSeqIndex = 0;

        // Loop through all possible matches in the buffer
        while(index_buffer>=index_min)
        {
            // Current matching Sequence size
            int intCurrSeqSize = 0;

            // Floating index
            int indexFloat = curr_index;
            int indexFloatBuffer = index_buffer;

            // Increase Sequence size
            // When bytes are equal
            while ( indexFloat<index_max
                    && (TabTrameReg[indexFloatBuffer++][r] == TabTrameReg[indexFloat++][r]))
            {
                intCurrSeqSize++;
            }

            // If CurrCalculated Size is bigger than the previous one, swap them
            if (intCurrSeqSize>intSeqSize)
            {
                intSeqSize = intCurrSeqSize;
                intSeqIndex = index_buffer;
            }

            // Decrease index in buffer
            index_buffer--;
        }

        if (intSeqSize > intMinSeqSize)
        {
            // A previously uncrunched pattern has been found
            // We need to:
            // - neg the length
            // - calculate the offset of previous maching Pattern in the buffer
            // - set bit Flag
            // - decrement Sequence Counter
            // - increment curr_index

            // length
            byte bytNegLength = (byte)((-intSeqSize) & 0xFF);
            arrCrunchedData[intCrunchedLength++] = bytNegLength;

            // Offset
            if (intBuffSize <= 0x100)
            {
                // Previous Offset
                BYTE bytePrevOffset = (BYTE)intSeqIndex;

                // Write 0x100 Offset
                arrCrunchedData[intCrunchedLength++] = bytePrevOffset;
            }
            else
            {
                // Previous Offset
                short shortPrevOffset = (short)intSeqIndex;

                // Write 0x400 Offset
                arrCrunchedData[intCrunchedLength++] = (byte)(shortPrevOffset & 0xFF);
                arrCrunchedData[intCrunchedLength++] = (byte)(shortPrevOffset>>8 & 0x3);
            }

            // Sequence Counter
            intSeqCounter--;

            // set bit Flag
            bsFlag |= 1<<intSeqCounter;

            // Inc index
            curr_index += intSeqSize;
        }
        else
        {
            // No Pattern found (put a character)
            // - put character
            // - unset bit Flag (nothing to do)
            // - decrement Sequence Counter
            // - increment curr_index

            // New character
            arrCrunchedData[intCrunchedLength++] = TabTrameReg[curr_index][r];

            // Sequence Counter
            intSeqCounter--;

            // Inc index
            curr_index++;
        }
    }

    // Insert Final Flag in flow
    arrCrunchedData[intOffsetFlag] = bsFlag;

    // return Crunched Array
    *intCrunLen = intCrunchedLength;
    return arrCrunchedData;
}


/********************************************************* !NAME! **************
* Nom : WriteYMEnd
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Termine le fichier YM
*
* Rsultat    : /
*
* Variables globales modifies : pfile, ps, psCmp
*
********************************************************** !0! ****************/
void WriteYMEnd( HWND hWnd )
{
    static BYTE HeaderYM[ 34 ];
    int FreqAYCPC = 1000000;

/*
En-tte fichier YM5 : (format BIG ENDIAN)

Offset  Size    Type        Comment 
0       4       DWORD       ID of YM5 format. ('YM5!') 
4       8       string[8]   Check String ('LeOnArD!') 
12      4       DWORD       Nb of valid VBL of the file 
16      4       DWORD       Song attributes (see bellow) 
20      2       WORD        Nb of digi-drum sample (can be 0) 
22      4       DWORD       YM2149 External frequency in Hz (1000000)
26      2       WORD        Player frequency in Hz (50Hz for almost player) 
28      4       DWORD       VBL number to loop the song. (0 is default)
32      2       WORD        Must be 0 for the moment
?       ?       NT-String   Name of the song 
?       ?       NT-String   Name of the author 
?       ?       NT-String   Comments (Name of the YM converter !) 
?       ?                   All YM2149 registers 
?       4       DWORD       End-File check ('End!') 

*/


    memset( HeaderYM, 0, sizeof( HeaderYM ) );
    memcpy( HeaderYM, ( BYTE * )"YM5!LeOnArD!", 12 );
    if ( NumTrame )
        {
        // Nb frames (0 - 0xFFFF pour le moment )
        HeaderYM[ 14 ] = ( BYTE )( NumTrame >> 8 );
        HeaderYM[ 15 ] = ( BYTE )NumTrame;
        // Attribut (frames entrelaces)
        HeaderYM[ 19 ] = YMEntrelace;
        // Frquence puce AY
        HeaderYM[ 22 ] = ( BYTE )( FreqAYCPC >> 24 );
        HeaderYM[ 23 ] = ( BYTE )( FreqAYCPC >> 16 );
        HeaderYM[ 24 ] = ( BYTE )( FreqAYCPC >> 8 );
        HeaderYM[ 25 ] = ( BYTE )FreqAYCPC;
        // Frquence player
        HeaderYM[ 27 ] = 50; // 50 hz
        fwrite( HeaderYM, sizeof( HeaderYM ), 1, FileYM );
        fprintf( FileYM, "YM Write%c", 0 );
        fprintf( FileYM, "PC-CPC%c", 0 );
        fprintf( FileYM, TITRE "%c", 0 );
        if ( YMEntrelace )
            {
            for ( int r = 0; r < 16; r++ )
                for ( int f = 0; f < NumTrame; f++ )
                    fwrite( &TabTrameReg[ f ][ r ], 1, 1, FileYM );
            }
        else
            fwrite( TabTrameReg, NumTrame * 16, 1, FileYM );

        fprintf( FileYM, "End!" );
        }
    wsprintf( MsgLog
            , "Sauvegarde YM termine. Nombre de trames crites : %d"
            , NumTrame
            );
    MessageBox( hWnd, MsgLog, "Sauvegarde YM", MB_OK );
    YMFileStarted = FALSE;
    fclose( FileYM );
    FileYM = NULL;

    /*******************
     * Create AYC file *
     *******************/
    FILE* FileAYC = fopen( "Crunched_tune.ayc", "wb+ " );
    if ( FileAYC )
        {
        // Process each Register
        int arrCrunLen[14];
        static BYTE arrCrunchedRegister[14][ 0x10000 ];
        BYTE arrBuffLen[14];
        
        for (BYTE curr_register=0;curr_register < 14; curr_register++)
            {                                
            // Crunch register Data 
            USHORT intCrunLen;
            USHORT intCrunLen400;
            BYTE* arrCrunchedData = doAYCCrunch( curr_register, (USHORT)NumTrame, 0x100, &intCrunLen);
            BYTE* arrCrunchedData400 = doAYCCrunch( curr_register, (USHORT)NumTrame, 0x400, &intCrunLen400);
                
            if ((intCrunLen400 + 0x300) < intCrunLen)
                {
                // Use a 0x400 bytes buffer rather than a 0x100 one

                // Buffer length
                arrBuffLen[curr_register] = 0x4;

                // Add Crunched register to the list
                memcpy( arrCrunchedRegister[curr_register], arrCrunchedData400, 0x10000 );
                arrCrunLen[curr_register] = intCrunLen400;                
                }
            else
                {
                // Use 0x100 Buffer

                // Buffer length
                arrBuffLen[curr_register] = 0x1;
            
                // Add Crunched register to the list
                memcpy( arrCrunchedRegister[curr_register], arrCrunchedData, 0x10000 );
                arrCrunLen[curr_register] = intCrunLen;                
                }
            }

        MessageBox( hWnd, "Writing file ...", "Sauvegarde AYC", MB_OK );

        /**
         * Write Header
         */
                
        // Time
        fwrite( &NumTrame, 2, 1, FileAYC );
        
        // Registers' offset
        int offsetBase = 2 + 2;
        int offsetPhysical = 2 + 16*3;
        int offset = offsetPhysical - offsetBase;
        fwrite( arrBuffLen, 1, 1, FileAYC );
        fwrite( &offset, 2, 1, FileAYC );
        for (int i=1;i<14;i++)
            {
            // get Relative Offset
            offsetPhysical += arrCrunLen[i-1];
            offsetBase += 3;
            offset = offsetPhysical - offsetBase;
            
            fwrite( arrBuffLen+i, 1, 1, FileAYC );
            fwrite( &offset, 2, 1, FileAYC );
            }
        
        // Add Dummy values for 14,15 Registers' Offset
        BYTE dummy = 0xFF;
        for (int b=0;b<6;b++) fwrite( &dummy, 1, 1, FileAYC );
            
        /**
         * Write Data
         */
        for (int j=0;j<14;j++)
            fwrite( arrCrunchedRegister[j], 1, arrCrunLen[j], FileAYC );

        fclose( FileAYC );
        }
    else
        {
        // Impossible de crer le fichier ?
        wsprintf( MsgLog, "Impossible de crer le fichier AYC" );
        Erreur( MsgLog );
        }
}


/********************************************************* !NAME! **************
* Nom : WriteYMStarted
********************************************************** !PATHS! *************
* !./V1!\!./V2!\!./V3!\!./V4!\Fonctions
********************************************************** !1! *****************
*
* Fichier     : !./FPTH\\/FLE!, ligne : !./LN!
*
* Description : Indique si un fichier YM est en cours de traitement
*
* Rsultat    : TRUE si en cours, FALSE sinon
*
* Variables globales modifies : 
*
********************************************************** !0! ****************/
BOOL WriteYMStarted( void )
{
    return( YMFileStarted );
}

#endif
